#include <sys/inotify.h> // inotify
#include <unistd.h>      // read, fork, execl
#include <limits.h>      // NAME_MAX
#include <stdio.h>
#include <stdlib.h>
#include <string.h>      // strerror
#include <errno.h>       // errno
#include <sys/types.h>   // waitpid
#include <sys/wait.h>    // waitpid

#define BUF_LEN (16 * (sizeof(struct inotify_event) + NAME_MAX + 1))
#define ERROR(...) { fprintf(stderr, __VA_ARGS__); exit(EXIT_FAILURE); }

int main(int argc, char *argv[]) {
    int fd, wd, pid, status;
    char buf[BUF_LEN] __attribute__ ((aligned(8)));
    ssize_t nb_events;
    
    if (argc < 3)
        ERROR("usage : %s [FILE TO WATCH] [PROC TO LAUNCH]\n", argv[0]);
    
    /* Create inotify instance */
    if ((fd = inotify_init()) == -1)
        ERROR("Error in inotify_init\n");
    
    /* add a watch for all events */
    if ((wd = inotify_add_watch(fd, argv[1], IN_ALL_EVENTS)) == -1)
        ERROR("Error : inotify_add_watch\n");
    
    /* wait for an event */
    if ((nb_events = read(fd, buf, BUF_LEN)) < 0)
        ERROR("Error in read\n");
    
    /* launch a script if an event happens */
    pid = fork();   // fork
    if (pid < 0) {
        ERROR("fork\n");
    } else if (pid > 0) {
        execl(argv[2], argv[2], "&", NULL); // exec proc
        ERROR("%s\n", strerror(errno));
    } else {
        waitpid(pid, &status, 0); // wait child
        if (status != 0)
            ERROR("%s\n", strerror(errno));
    }

    /* remove the watch for all events */
    (void) inotify_rm_watch(fd, wd);
    (void) close(fd);
    
    exit(EXIT_SUCCESS);
}
